/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import "MyInspector.h"
#import <Cocoa/Cocoa.h>
#import "WinCtrl.h"

#define  PanelWidthDef    250
#define  PanelHeightDef   200
#define  BtnWidth         64
#define  BtnHeight        32
#define  Margin           10

@implementation MyInspector

static id defaultInstance = nil;

+ (id)defaultInstance
{
    if (defaultInstance == nil)
        defaultInstance = [[self alloc] init];
    [defaultInstance activate];
    return defaultInstance;
}

/* Local Method */
- (void)panelSetting
{
    id  button[2];
    NSBundle *bundle;
    NSString *localstr;
    NSRect rect;
    float wid, hgt;
    int i;

    bundle = [NSBundle bundleForClass:[self class]];
    rect = NSMakeRect(300, 300, PanelWidthDef, PanelHeightDef);
    panel = [[NSPanel alloc] initWithContentRect:rect
        styleMask: (NSTitledWindowMask | NSClosableWindowMask | NSResizableWindowMask)
        backing:NSBackingStoreBuffered
        defer:YES];
    [panel setReleasedWhenClosed:NO];
    [panel setMinSize:NSMakeSize(PanelWidthDef, PanelHeightDef)];
    [panel setFrameAutosaveName:@"Inspector Panel"];
    localstr = [bundle localizedStringForKey:@"Inspector" value:@"" table:nil];
    [panel setTitle: localstr];
    [panel setDelegate: self];
    isClosed = YES;

    for (i = 0; i < 2; i++) {
        button[i] = [[NSButton alloc] initWithFrame:
            NSMakeRect(Margin+(Margin+BtnWidth)*i, Margin, BtnWidth, BtnHeight)];
        [button[i] autorelease];
        [[panel contentView] addSubview:button[i]];
        [button[i] setAutoresizesSubviews:YES];
        [button[i] setAutoresizingMask:(NSViewMaxXMargin | NSViewMaxYMargin)];
        [button[i] setTarget:self];
    }
    localstr = [bundle localizedStringForKey:@"Shrink" value:@"" table:nil];
    [button[0] setTitle: localstr];
    [button[0] setAction:@selector(shrink:)];
    localstr = [bundle localizedStringForKey:@"All" value:@"" table:nil];
    [button[1] setTitle: localstr];
    [button[1] setAction:@selector(shrinkAll:)];

    rect = [panel frame];
    wid = rect.size.width - Margin*2;
    hgt = rect.size.height - (Margin + BtnHeight) * 2;
    text = [[NSTextField alloc] initWithFrame:
        NSMakeRect(Margin, BtnHeight*2 - Margin, wid, hgt)];
    [text setSelectable:NO];
    [text setBezeled:YES];
    [[panel contentView] addSubview: [text autorelease]];
    [text setAutoresizesSubviews:YES];
    [text setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];
}

- (id)init
{
    id center;

    [super init];
    [self panelSetting];
    center = [NSNotificationCenter defaultCenter];
    [center addObserver:self selector:@selector(showMain:)
        name:NSWindowDidBecomeMainNotification object:nil];
    [center addObserver:self selector:@selector(windowClosed:)
        name:NSWindowWillCloseNotification object:nil];
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [panel release];
    [super dealloc];
}

- (void)activate
{
    [panel setFloatingPanel:YES];
    [panel makeKeyAndOrderFront: self];
}

/* Local Method */
- (void)showInfo:(id)obj
{
    NSString *log = @"";
    if ([obj isKindOfClass:[NSWindow class]]) {
        id ctrl = [obj delegate];
        if ([ctrl respondsToSelector:@selector(attributesOfImage)])
            log = [ctrl attributesOfImage];
    }
    [text setStringValue: log];
}

- (void)showMain:(NSNotification *)aNotification
{
    if (isClosed) return;
    [self showInfo:[aNotification object]];
}

- (void)windowClosed:(NSNotification *)aNotification
{
    if ([[aNotification object] isEqual:[NSApp mainWindow]])
        [text setStringValue:@""];
}

- (void)shrink:(id)sender
{
    id obj = [NSApp mainWindow];
    if (obj) {
        [[obj delegate] shrink:nil];
        [self showInfo:obj];
    }
}

- (void)shrinkAll:(id)sender
{
    id obj = [NSApp mainWindow];
    if (obj) {
        [[NSNotificationCenter defaultCenter]
            postNotificationName:ShrinkAllNotification object:self];
        [self showInfo:obj];
    }
}

/* Delegate Message */
- (void)windowDidBecomeKey:(NSNotification *)aNotification
{
    isClosed = NO;
    [self showInfo:[NSApp mainWindow]];
}

/* Delegate Message */
- (BOOL)windowShouldClose:(id)sender
{
    isClosed = YES;
    return YES;
}

@end
