/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import "MyViewerCtrl.h"
#import <Cocoa/Cocoa.h>
#import "WinCtrl.h"
#import "MyInspector.h"

static NSString *AutoResizeKey = @"AutoResize";

@implementation MyViewerCtrl

/* Local Method */
- (void)setupMainMenu
{
    id <NSMenuItem>  topitem, item;
    NSMenu *menu, *submenu;
    NSString *str, *appname;
    NSBundle *bundle = [NSBundle mainBundle];

    appname = [bundle objectForInfoDictionaryKey:@"CFBundleName"];
    if (appname == nil)
        appname = @"MyViewer";
    menu = [[[NSMenu alloc] initWithTitle:appname] autorelease];
    str = NSLocalizedString(@"File", "File");
    topitem = [menu addItemWithTitle:str
        action:@selector(submenuAction:) keyEquivalent:@""];
    submenu = [[[NSMenu alloc] initWithTitle:str] autorelease];
    [menu setSubmenu:submenu forItem:topitem];
    str = NSLocalizedString(@"Open File...", "Open File");
    item = [submenu addItemWithTitle:str
        action:@selector(openFile:) keyEquivalent:@"o"];
    [item setTarget:self];
    str = NSLocalizedString(@"Inspector...", "Inspector");
    item = [submenu addItemWithTitle:str
        action:@selector(activateInspector:) keyEquivalent:@"i"];
    [item setTarget:self];
    [submenu addItem:[NSMenuItem separatorItem]];
    str = NSLocalizedString(@"Auto Resize", "Auto Resize");
    item = [submenu addItemWithTitle:str
        action:@selector(toggleAutoResize:) keyEquivalent:@""];
    [item setTarget:self];
    [item setState:([WinCtrl autoResize] ? NSOnState : NSOffState)];
    [submenu addItem:[NSMenuItem separatorItem]];
    str = NSLocalizedString(@"Hide %@", "Hide");
    item = [submenu addItemWithTitle:
        [NSString stringWithFormat:str, appname]
        action:@selector(hide:) keyEquivalent:@"h"];
    str = NSLocalizedString(@"Quit %@", "Quit");
    item = [submenu addItemWithTitle:
        [NSString stringWithFormat:str, appname]
        action:@selector(terminate:) keyEquivalent:@"q"];
    // If the target of a menu item is nil, it means First Responder.
    [NSApp setMainMenu:menu];
}

- (void)openFile:(id)sender
{
    NSArray *fileTypes = [NSImage imageFileTypes];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    NSArray *filesToOpen;
    NSEnumerator *enumerator;
    NSString *aFile;
    int result;

    [oPanel setAllowsMultipleSelection:YES];
    result = [oPanel runModalForDirectory:NSHomeDirectory()
        file:nil types:fileTypes];
    if (result != NSOKButton)
        return; /* Cancel */
    filesToOpen = [oPanel filenames];
    enumerator = [filesToOpen objectEnumerator];
    while ((aFile = [enumerator nextObject]) != nil)
        (void)[[WinCtrl alloc] initWithFile:aFile];
}

- (void) activateInspector:(id)sender
{
    static Class inspectorClass = Nil;
    NSBundle *bundle;
    NSString *path;

    if (inspectorClass == Nil) {
        path = [[NSBundle mainBundle]
                   pathForResource:@"Inspector" ofType:@"bundle"];
        if ((bundle = [NSBundle bundleWithPath: path]) == nil)
            return; // ERROR!
        inspectorClass = [bundle classNamed:@"MyInspector"];
    }
    (void)[inspectorClass defaultInstance];
}

- (void)toggleAutoResize:(id)sender
{
    BOOL newState = ([sender state] != NSOnState);
    [WinCtrl setAutoResize: newState];
    [sender setState:(newState ? NSOnState : NSOffState)];
    [[NSUserDefaults standardUserDefaults]
        setBool:newState forKey:AutoResizeKey];
}

/* Delegate Message */
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [WinCtrl setAutoResize:
        [[NSUserDefaults standardUserDefaults] boolForKey:AutoResizeKey]];
    [self setupMainMenu];
}

/* Delegate Message */
- (void)applicationWillTerminate:(NSNotification *)aNotification
{
    [[NSUserDefaults standardUserDefaults] synchronize];
}

@end
