/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import "WinCtrl.h"
#import <Cocoa/Cocoa.h>

#define  ImageSizeMIN   32
#define  TitleBarHeight 22

/*extern*/ NSString *ShrinkAllNotification = @"ShrinkAllNotification";

static BOOL autoResize = NO;
static NSSize screenSize = { 1028.0, 768.0 };

@implementation WinCtrl

+ (void)initialize
{
    NSScreen *screen = [[NSScreen screens] objectAtIndex:0];
    screenSize = [screen visibleFrame].size;
    screenSize.height -= TitleBarHeight;
}

+ (BOOL)autoResize { return autoResize; }
+ (void)setAutoResize:(BOOL)flag {
    autoResize = flag;
}

/* Local Method */
- (void)windowSetUp
{
    static int wincount = 0;
    float cas = (wincount++ & 07) * 20.0;
    id imageview;
    NSRect winrect, vwrect;
    float x, y;

    winrect.size.width = (int)(originalSize.width * mag);
    winrect.size.height = (int)(originalSize.height * mag);
    x = 100.0 + cas;
    y = 150.0 + cas;
    if (x + winrect.size.width > screenSize.width)
        x = screenSize.width - winrect.size.width;
    if (y + winrect.size.height > screenSize.height)
        y = screenSize.height - winrect.size.height;
    winrect.origin = NSMakePoint(x, y);
    window = [[NSWindow alloc] initWithContentRect:winrect
        styleMask: (NSTitledWindowMask | NSClosableWindowMask)
        backing:NSBackingStoreBuffered
        defer:YES];
    [window setReleasedWhenClosed:YES];
    vwrect.size = originalSize;
    vwrect.origin = NSZeroPoint;
    imageview = [[NSImageView alloc] initWithFrame:vwrect];
    [imageview setImage:docImage];
    [imageview setEditable:NO];
    [imageview setImageScaling:YES];
    [imageview setFrameSize: winrect.size];
    [window setContentView:imageview];
    [imageview release];
}

- (id)initWithFile:(NSString *)path
{
    NSImageRep *rep;
    int x;

    [super init];
    if ((docImage = [[NSImage alloc] initWithContentsOfFile:path]) == nil) {
        [self autorelease];
        return nil;
    }
    filename = [path retain];

    originalSize = [docImage size];
    rep = [docImage bestRepresentationForDevice:nil];
    x = [rep pixelsWide];
    if (x != NSImageRepMatchesDevice && x != originalSize.width) {
        originalSize = NSMakeSize(x, [rep pixelsHigh]);
        [docImage setSize:originalSize];
    }
    mag = 1.0;
    if (autoResize) {
        double wr = screenSize.width / originalSize.width;
        double hr = screenSize.height / originalSize.height;
        double ratio = (wr < hr) ? wr : hr;
        if (ratio < 1.0)
            mag = ratio;
    }
    [self windowSetUp];
    [[NSNotificationCenter defaultCenter] addObserver:self
        selector:@selector(shrink:) name:ShrinkAllNotification object:nil];
    [window setTitleWithRepresentedFilename:filename];
    [window setDelegate:self];
    [window makeKeyAndOrderFront:self];
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [window setDelegate:nil];
    [filename release];
    [docImage release];
    [super dealloc];
}

- (NSString *)attributesOfImage
{
    static NSString *fnamestr, *sizestr, *magstr;
    NSSize sz;

    if (fnamestr == nil) {
        fnamestr = [NSLocalizedString(@"Filename", "filename") retain];
        sizestr = [NSLocalizedString(@"Size", "Size") retain];
        magstr = [NSLocalizedString(@"Magnification", "Magnification") retain];
    }
    sz = [docImage size];
    return [NSString stringWithFormat:
        @"%@: %@\n%@: %d x %d\n%@: %.1lf%%",
        fnamestr, [filename lastPathComponent],
        sizestr, (int)sz.width, (int)sz.height, magstr, mag*100.0];
}

- (void)shrink:(NSNotification *)notification
{
    id view = [window contentView];
    NSSize sz = [view frame].size;
    NSRect rect = [window frame];
    int wt;

    if (sz.width < ImageSizeMIN || sz.height < ImageSizeMIN)
        return;
    mag *= 0.5;
    sz.width = (int)(originalSize.width * mag);
    sz.height = (int)(originalSize.height * mag);
    [view setFrameSize:sz];
    wt = rect.origin.y + rect.size.height;
    rect.size = sz;
    rect.origin.y = wt - rect.size.height;
    [window setFrame:rect display:YES];
    [window setDocumentEdited:YES];
}

/* Delegate Message */
- (BOOL)windowShouldClose:(id)sender
{
    static NSString *warnstr, *closestr, *okstr, *cancelstr;
    if (warnstr == nil) {
        warnstr = [NSLocalizedString(@"File %@ is edited.", "Edited") retain];
        closestr = [NSLocalizedString(@"Close the window?", "Close?") retain];
        okstr = [NSLocalizedString(@"OK", "OK") retain];
        cancelstr = [NSLocalizedString(@"Cancel", "Cancel") retain];
    }
    if ([window isDocumentEdited])
        return NSRunAlertPanel(closestr, warnstr, okstr, cancelstr,
            nil, [filename lastPathComponent]) == NSAlertDefaultReturn;
    return YES;
}

/* Delegate Message */
- (void)windowWillClose:(NSNotification *)aNotification
{
    [self release];
}

@end
