/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import "Philosopher.h"
#import <Foundation/Foundation.h>
#import <stdlib.h>
#import <unistd.h> // for getpid()

static NSLock *sharedLock = nil;

@implementation Philosopher

+ (void)initialize
{
    if (sharedLock == nil) {
        sharedLock = [[NSLock alloc] init];
        srandom(getpid());
    }
}

- (id)initWithLeft:(id <NSLocking>)llock right:(id <NSLocking>)rlock;
{
    [super init];
    leftLock = llock;
    rightLock = rlock;
    tag = 0;
    state = phil_Meditation;
    return self;
}

- (void)setTag:(int)val {
    tag = val;
}

static void waitAMoment(int r) {
    int n = r * r;
    while (n-- > 0) ;
}

- (void)start:(id)sender
{
    int rnd = tag * 50;

    while ( 1 ) {
        id pool = [[NSAutoreleasePool alloc] init];
        [leftLock lock];
        state = phil_WaitRight;
        waitAMoment(rnd);
        [rightLock lock];

        state = phil_Eating;
        [sharedLock lock];
        // random() and printf() are thread-unsafe.
        rnd = random() % 1000 + 1;
        printf("Philosopher #%d\n", tag);
        [sharedLock unlock];

        [leftLock unlock];
        [rightLock unlock];
        state = phil_Meditation;
        waitAMoment(rnd);
        [pool release];
    }
}

@end
