/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import "UroCtrl.h"
#import <Cocoa/Cocoa.h>
#import "UroPiece.h"
#import "UroView.h"

#define  NumberOfClients    15

@implementation UroCtrl

/* Local Method */
- (void)clearBoard
{
    int x, y;
    for (y = 0; y < BHeight; y++)
        for (x = 0; x < BWidth; x++)
            uBoard[y][x] = nil;
}

- (id)init
{
    int i;
    [super init];
    proxies = [[NSMutableArray alloc] initWithCapacity:1];
    uBoard = (id **)malloc(sizeof(id *) * BHeight);
    for (i = 0; i < BHeight; i++)
        uBoard[i] = (id *)malloc(sizeof(id) * BWidth);
    [self clearBoard];
    uView = [[UroView alloc]
        initWithGrid:GridSize width:BWidth height:BHeight];
    [uView setBoard: uBoard];
    inAction = NO;
    return self;
}

- (void)dealloc
{
    int i;
    for (i = 0; i < BHeight; i++)
        free((void *)uBoard[i]);
    free((void *)uBoard);
    [uView release];
    [proxies release];
    [super dealloc];
}

/* Local Method */
- (void)setupMainMenu
{
    id <NSMenuItem>  item;
    NSMenu *menu, *submenu;

    menu = [[[NSMenu alloc] initWithTitle: AppName] autorelease];
    item = [menu addItemWithTitle:@"Action"
        action:@selector(submenuAction:) keyEquivalent:@""];
    submenu = [[[NSMenu alloc] initWithTitle:@"Action"] autorelease];
    [menu setSubmenu: submenu forItem:item];
    item = [submenu addItemWithTitle:@"New Board"
        action:@selector(resetBoard:) keyEquivalent:@"n"];
    [item setTarget:self];
    item = [submenu addItemWithTitle:@"Quit Urouro"
        action:@selector(terminate:) keyEquivalent:@"q"];
    // if target is nil, the message is sent to the first responder.
    [NSApp setMainMenu:menu];
}

/* Local Method */
- (void)windowSetUp
{
    NSRect rect = [uView frame];
    rect.origin = NSMakePoint(120.0, 150.0);
    uWindow = [[NSWindow alloc] initWithContentRect:rect
        styleMask: NSTitledWindowMask
        backing:NSBackingStoreBuffered defer:YES];
    [uWindow setReleasedWhenClosed:YES];
    [uWindow setTitle: AppName];
    [uWindow setContentView:uView];
}

/* Local Method */
- (void)clientSetUp
{
    NSPort *port1;
    NSPort *port2;
    NSArray *portArray;
    NSConnection *conn;

    port1 = [NSPort port];
    port2 = [NSPort port];
    conn = [[NSConnection alloc]
        initWithReceivePort:port1 sendPort:port2];
    if (conn == nil)
        return;
    [conn setRootObject:self];
    portArray = [NSArray arrayWithObjects:port2, port1, nil];

    [NSThread detachNewThreadSelector:@selector(connectNewClient:)
        toTarget:[UroPiece class] withObject:portArray];
}

- (void)registerAsClient:(id)client {
    [client setProtocolForProxy:@protocol(UroClient)];
    [proxies addObject:client];
}

- (BOOL)moveFrom:(UroPoint)loc to:(UroPoint)newloc
{
    int xmin, xmax, ymin, ymax;
    NSRect rect;

    if (!inAction || uBoard[newloc.y][newloc.x])
        return NO;
    uBoard[newloc.y][newloc.x] = uBoard[loc.y][loc.x];
    uBoard[loc.y][loc.x] = nil;
    if (loc.x > newloc.x)
        xmin = newloc.x, xmax = loc.x;
    else
        xmin = loc.x, xmax = newloc.x;
    if (loc.y > newloc.y)
        ymin = newloc.y, ymax = loc.y;
    else
        ymin = loc.y, ymax = newloc.y;
    rect = NSMakeRect(xmin*GridSize, ymin*GridSize,
        (xmax-xmin+1)*GridSize, (ymax-ymin+1)*GridSize);

    [uView displayRect:rect];
    return YES;
}

- (void)resetBoard:(id)sender
{
    int i;

    inAction = NO;
    for (i = [proxies count] - 1; i >= 0; i--)
        [(UroPiece *)[proxies objectAtIndex:i] stop];
    [self clearBoard];
    for (i = [proxies count] - 1; i >= 0; i--) {
        UroPoint loc;
        UroPiece *obj = [proxies objectAtIndex:i];
        do {
            loc = [obj reset];
        }while (uBoard[loc.y][loc.x]);
        uBoard[loc.y][loc.x] = obj;
    }
    [uView display];
    inAction = YES;
    for (i = [proxies count] - 1; i >= 0; i--)
        [(UroPiece *)[proxies objectAtIndex:i] start];
}

/* Application Delegate Message */
- (void)applicationDidFinishLaunching:(NSNotification *)note
{
    int i;

    [self setupMainMenu];
    [self windowSetUp];
    [UroPiece setBoardWidth:BWidth height:BHeight];

    for (i = 0; i < NumberOfClients; i++)
        [self clientSetUp];
    [uWindow makeKeyAndOrderFront:self];
}

@end
