/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import "UroPiece.h"
#import <Foundation/Foundation.h>
#import <AppKit/NSColor.h>
#import <stdlib.h>
#import <unistd.h>
#import "UroCtrl.h"

#define SpeedFactor  0.2

@implementation UroPiece

static NSLock *rndLock = nil;
static int bWidth, bHeight;

+ (void)initialize
{
    if (rndLock == nil) {
        srandom((unsigned int)getpid());
        rndLock = [[NSLock alloc] init];
    }
}

+ (int)random:(int)range
{
    int v;
    [rndLock lock];
    v = random() % range;
    [rndLock unlock];
    return v;
}

+ (void)setBoardWidth:(int)w height:(int)h {
    bWidth = w, bHeight = h;
}

+ (void)connectNewClient:(id)ports
{
    id pool = [[NSAutoreleasePool alloc] init];
    NSConnection *conn = [NSConnection
        connectionWithReceivePort:[ports objectAtIndex:0]
        sendPort:[ports objectAtIndex:1]];
    if (conn) {
        id ctrl, client;
        ctrl = [conn rootProxy];
        [ctrl setProtocolForProxy:@protocol(UroServices)];
        client = [[self alloc] initWithController: ctrl];
        [ctrl registerAsClient:client];
        [client release];
        [[NSRunLoop currentRunLoop] run];
    }
    [pool release];
    [NSThread exit];
}

- (id)initWithController:(id)obj
{
    [super init];
    controller = obj;
    lastDirection = [[self class] random:Directions];
    color = [NSColor
        colorWithCalibratedHue:[[self class] random:12] / 12.0
        saturation: ([[self class] random:2] ? 1.0 : 0.6)
        brightness:1.0 alpha:1.0];
    inAction = NO;
    return self;
}

- (NSColor *)color { return color; }

- (UroPoint)reset
{
    location.x = [[self class] random:bWidth];
    location.y = [[self class] random:bHeight];
    return location;
}

- (void)move:(id)sender
{
    int d;
    BOOL ok;
    UroPoint newloc;

    d = [[self class] random: Directions+4];
    if (d >= Directions) d = lastDirection;
    for (ok = NO; ; d = (d+1) % Directions) {
        newloc = location;
        switch (d) {
        case d_Up:    if (++newloc.y < bHeight) ok = YES;
            break;
        case d_Down:  if (--newloc.y >= 0) ok = YES;
            break;
        case d_Right: if (++newloc.x < bWidth) ok = YES;
            break;
        case d_Left:  if (--newloc.x >= 0) ok = YES;
            break;
        }
        if (ok) break;
    }
    lastDirection = d;
    if ([controller moveFrom:location to:newloc])
        location = newloc;
    if (inAction)
        [self start];
}

- (oneway void)start {
    NSTimeInterval delay = [[self class] random:15] / 10.0 + 0.5;
    inAction = YES;
    [self performSelector:@selector(move:)
        withObject:self afterDelay: delay * SpeedFactor];
}

- (oneway void)stop {
    [[self class] cancelPreviousPerformRequestsWithTarget:self
        selector:@selector(move:) object:self];
    inAction = NO;
}

@end
