/*** Copyright: OGIHARA Takeshi, March 2006 ***/

#import <Foundation/Foundation.h>

@interface Creature : NSObject
{
    NSString *name;
    int      hitPoint;
}
- (void)setHitPoint:(int)val;
- (void)sufferDamage:(int)val;
- (NSString *)description;
- (void)observeValueForKeyPath:(NSString *)keyPath
    ofObject:(id)object change:(NSDictionary *)change
    context:(void *)context;
@end

@implementation Creature

- (void)setHitPoint:(int)val { hitPoint = val; }

- (void)sufferDamage:(int)val {
    hitPoint = (hitPoint > val) ? (hitPoint - val) : 0;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"<<%@, %@, HP=%d>>",
               NSStringFromClass([self class]), name, hitPoint];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
    ofObject:(id)object change:(NSDictionary *)change
    context:(void *)context
{
    NSLog(@"--- Received by %@ ---\n", self);
    NSLog(@"Object=%@, Path=%@\n", object, keyPath);
    NSLog(@"Change=%@\n", change);
}
@end

@interface Person : Creature
@end

@implementation Person
@end

@interface Monster : Creature {
    Person   *master;
}
@end

@implementation Monster
@end

int main(void)
{
    id pool = [[NSAutoreleasePool alloc] init];
    id p1 = [[Person alloc] init];
    id p2 = [[Person alloc] init];
    id dragon = [[Monster alloc] init];
    int opt = NSKeyValueObservingOptionOld | NSKeyValueObservingOptionNew;

    [p1 setValue:@"Taro" forKey:@"name"];
    [p1 setHitPoint:300];
    [p2 setValue:@"Jiro" forKey:@"name"];
    [p2 setHitPoint:140];
    [dragon setValue:@"Choco" forKey:@"name"];
    [dragon setHitPoint:200];
    [dragon setValue:p1 forKey:@"master"];
    [dragon addObserver:dragon forKeyPath:@"master.name"
            options:opt context:NULL];                        // A
    [p1 addObserver:p2 forKeyPath:@"hitPoint"
            options:opt context:NULL];                        // B
    
    [dragon setValue:@"Tommy" forKeyPath:@"master.name"];     // 1
    [p1 setHitPoint:500];                                     // 2
    [p1 sufferDamage:400];                                    // 3
    [dragon setValue:p2 forKey:@"master"];                    // 4
    [p2 setValue:@"John" forKey:@"name"];                     // 5
    [dragon removeObserver:dragon forKeyPath:@"master.name"]; // 6
    [p2 setValue:@"Kenshiro" forKey:@"name"];                 // 7

    [pool release];
    return 0;
}
